"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeGraphQLServer = void 0;
require("reflect-metadata");
const lodash_flowright_1 = __importDefault(require("lodash.flowright"));
const apollo_server_micro_1 = require("apollo-server-micro");
const buildSchema_1 = require("type-graphql/dist/utils/buildSchema");
const typeorm_1 = require("./adapters/typeorm");
const initMiddleware_1 = require("./utils/initMiddleware");
class TypeGraphQLServer extends apollo_server_micro_1.ApolloServer {
    constructor({ schema, middleware, typeormOptions, path, disableHealthCheck, onHealthCheck, ...rest }) {
        super({ ...rest, schema: buildSchema_1.buildSchemaSync(schema) });
        this.typeormOptions = typeormOptions;
        this.buildSchema = schema;
        this.serverRegistration = { path, disableHealthCheck, onHealthCheck };
        this.middleware = middleware;
    }
    createHandler() {
        const handler = this.typeormOptions
            ? async (req, res) => {
                await typeorm_1.connectTypeorm(this.typeormOptions);
                return super.createHandler(this.serverRegistration)(req, res);
            }
            : super.createHandler(this.serverRegistration);
        const middleware = this.createMiddleware(this.middleware);
        return middleware ? lodash_flowright_1.default(...middleware)(handler) : handler;
    }
    createMiddleware(middleware) {
        if (!(middleware === null || middleware === void 0 ? void 0 : middleware.length)) {
            return;
        }
        return middleware.map((middlewareFn) => {
            const wrappedMiddleware = initMiddleware_1.initMiddleware(middlewareFn);
            return (handler) => async (req, res) => {
                await wrappedMiddleware(req, res);
                return handler(req, res);
            };
        });
    }
}
exports.TypeGraphQLServer = TypeGraphQLServer;
